/*
 * $Id: phk_binheap.h,v 1.1 2006/09/09 12:10:09 phk Exp $
 *
 * Binary Heap API (see: http://en.wikipedia.org/wiki/Binary_heap)
 *
 * XXX: doesn't scale back the array of pointers when items are deleted.
 */

/* Public Interface --------------------------------------------------*/

struct phkbinheap;

typedef int phkbinheap_cmp_t(void *priv, void *a, void *b);
	/*
	 * Comparison function.
 	 * Should return true if item 'a' should be closer to the root
	 * than item 'b'
	 */

typedef void phkbinheap_update_t(void *priv, void *a, unsigned newidx);
	/*
	 * Update function (optional)
	 * When items move in the tree, this function gets called to
	 * notify the item of its new index.
	 * Only needed if deleting non-root items.
	 */

struct phkbinheap *PhkBinHeapNew(void *priv, phkbinheap_cmp_t, phkbinheap_update_t);
	/*
	 * Create Binary tree
	 * 'priv' is passed to cmp and update functions.
 	 */

int PhkBinHeapInsert(struct phkbinheap *, void *);
	/*
	 * Insert an item
	 */

void PhkBinHeapDelete(struct phkbinheap *, unsigned idx);
	/*
	 * Delete an item
	 * The root item has 'idx' zero
	 */

void *PhkBinHeapRoot(struct phkbinheap *);
	/*
	 * Return the root item
	 */

